@@ -127,6 +127,8 @@ enum {
 	N_SIGNALS
 };
 
+static const cairo_user_data_key_t average_color_key;
+
 static guint signals[N_SIGNALS] = { 0 };
 
 G_DEFINE_TYPE (GnomeBG, gnome_bg, G_TYPE_OBJECT)
@@ -136,7 +138,8 @@ static cairo_surface_t *make_root_pixmap     (GdkScreen  *screen,
                                               gint        height);
 
 /* Pixbuf utils */
-static guint32    pixbuf_average_value (GdkPixbuf  *pixbuf);
+static void       pixbuf_average_value (GdkPixbuf  *pixbuf,
+                                        GdkRGBA    *result);
 static GdkPixbuf *pixbuf_scale_to_fit  (GdkPixbuf  *src,
 					int         max_width,
 					int         max_height);
@@ -305,7 +308,7 @@ gnome_bg_load_from_preferences (GnomeBG   *bg,
 	/* Filename */
 	filename = NULL;
 	tmp = g_settings_get_string (settings, BG_KEY_PICTURE_URI);
-	if (*tmp != '\0') {
+	if (tmp && *tmp != '\0') {
 		filename = g_filename_from_uri (tmp, NULL, NULL);
 		if (filename != NULL && g_file_test (filename, G_FILE_TEST_EXISTS) == FALSE) {
 			g_free (filename);
@@ -761,8 +764,7 @@ draw_color_area (GnomeBG *bg,
 
 static void
 draw_color (GnomeBG *bg,
-	    GdkPixbuf *dest,
-	    GdkScreen *screen)
+	    GdkPixbuf *dest)
 {
 	GdkRectangle rect;
 	rect.x = 0;
@@ -925,8 +927,7 @@ draw_image_for_thumb (GnomeBG       *bg,
 
 static void
 draw_once (GnomeBG   *bg,
-	   GdkPixbuf *dest,
-	   GdkScreen *screen)
+	   GdkPixbuf *dest)
 {
 	GdkRectangle rect;
 	GdkPixbuf   *pixbuf;
@@ -990,9 +991,9 @@ gnome_bg_draw (GnomeBG *bg,
 			draw_each_monitor (bg, dest, screen);
 		}
 	} else {
-		draw_color (bg, dest, screen);
+		draw_color (bg, dest);
 		if (bg->placement != G_DESKTOP_BACKGROUND_STYLE_NONE) {
-			draw_once (bg, dest, screen);
+			draw_once (bg, dest);
 		}
 	}
 }
@@ -1071,6 +1072,7 @@ gnome_bg_create_surface (GnomeBG	    *bg,
 {
 	int pm_width, pm_height;
 	cairo_surface_t *surface;
+	GdkRGBA average;
 	cairo_t *cr;
 	
 	g_return_val_if_fail (bg != NULL, NULL);
@@ -1099,6 +1101,10 @@ gnome_bg_create_surface (GnomeBG	    *bg,
 	cr = cairo_create (surface);
 	if (!bg->filename && bg->color_type == G_DESKTOP_BACKGROUND_SHADING_SOLID) {
 		gdk_cairo_set_source_color (cr, &(bg->primary));
+		average.red = bg->primary.red / 255.0;
+		average.green = bg->primary.green / 255.0;
+		average.blue = bg->primary.blue / 255.0;
+		average.alpha = 1.0;
 	}
 	else {
 		GdkPixbuf *pixbuf;
@@ -1107,6 +1113,7 @@ gnome_bg_create_surface (GnomeBG	    *bg,
 					 width, height);
 		gnome_bg_draw (bg, pixbuf, gdk_window_get_screen (window), is_root);
 		gdk_cairo_set_source_pixbuf (cr, pixbuf, 0, 0);
+		pixbuf_average_value (pixbuf, &average);
 		g_object_unref (pixbuf);
 	}
 
@@ -1114,6 +1121,10 @@ gnome_bg_create_surface (GnomeBG	    *bg,
 	
 	cairo_destroy (cr);
 
+	cairo_surface_set_user_data (surface, &average_color_key,
+	                             gdk_rgba_copy (&average),
+	                             (cairo_destroy_func_t) gdk_rgba_free);
+
 	return surface;
 }
 
@@ -1141,11 +1152,13 @@ gnome_bg_is_dark (GnomeBG *bg,
 	}
 	pixbuf = get_pixbuf_for_size (bg, -1, width, height);
 	if (pixbuf) {
-		guint32 argb = pixbuf_average_value (pixbuf);
-		guchar a = (argb >> 24) & 0xff;
-		guchar r = (argb >> 16) & 0xff;
-		guchar g = (argb >>  8) & 0xff;
-		guchar b = (argb >>  0) & 0xff;
+		GdkRGBA argb;
+
+		pixbuf_average_value (pixbuf, &argb);
+		guchar a = argb.alpha * 0xff;
+		guchar r = argb.red * 0xff;
+		guchar g = argb.green * 0xff;
+		guchar b = argb.blue * 0xff;
 		
 		color.red = (color.red * (0xFF - a) + r * 0x101 * a) / 0xFF;
 		color.green = (color.green * (0xFF - a) + g * 0x101 * a) / 0xFF;
@@ -1308,7 +1321,7 @@ gnome_bg_create_thumbnail (GnomeBG               *bg,
 	
 	result = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, dest_width, dest_height);
 	
-	draw_color (bg, result, screen);
+	draw_color (bg, result);
 	
 	if (bg->placement != G_DESKTOP_BACKGROUND_STYLE_NONE) {
 		thumb = create_img_thumbnail (bg, factory, screen, dest_width, dest_height, -1);
@@ -1436,6 +1449,7 @@ gnome_bg_set_root_pixmap_id (GdkScreen       *screen,
 	Atom     type;
 	Display *display;
 	int      screen_num;
+	GdkRGBA *average;
 
 	screen_num = gdk_screen_get_number (screen);
 	data_esetroot = NULL;
@@ -1471,6 +1485,33 @@ gnome_bg_set_root_pixmap_id (GdkScreen       *screen,
 			 gdk_x11_get_xatom_by_name ("_XROOTPMAP_ID"), XA_PIXMAP,
 			 32, PropModeReplace,
 			 (guchar *) &pixmap_id, 1);
+
+	average = cairo_surface_get_user_data (surface, &average_color_key);
+	if (average != NULL) {
+		gchar *string;
+
+		string = gdk_rgba_to_string (average);
+
+		/* X encodes string lists as one big string with a nul
+		 * terminator after each item in the list.  That's why
+		 * the strlen has to be given; scanning for nul would
+		 * only find the first item.
+		 *
+		 * For now, we only want to set a single string.
+		 * Fortunately, since this is C, it comes with its own
+		 * nul and we can just give strlen + 1 for the size of
+		 * our "list".
+		 */
+		XChangeProperty (display, RootWindow (display, screen_num),
+		                 gdk_x11_get_xatom_by_name ("_GNOME_BACKGROUND_REPRESENTATIVE_COLORS"),
+		                 XA_STRING, 8, PropModeReplace,
+		                 (guchar *) string, strlen (string) + 1);
+		g_free (string);
+	} else {
+		/* Could happen if we didn't create the surface... */
+		XDeleteProperty (display, RootWindow (display, screen_num),
+		                 gdk_x11_get_xatom_by_name ("_GNOME_BACKGROUND_REPRESENTATIVE_COLORS"));
+	}
 }
 
 /**
@@ -2318,8 +2359,9 @@ clear_cache (GnomeBG *bg)
 }
 
 /* Pixbuf utilities */
-static guint32
-pixbuf_average_value (GdkPixbuf *pixbuf)
+static void
+pixbuf_average_value (GdkPixbuf *pixbuf,
+                      GdkRGBA   *result)
 {
 	guint64 a_total, r_total, g_total, b_total;
 	guint row, column;
@@ -2328,6 +2370,7 @@ pixbuf_average_value (GdkPixbuf *pixbuf)
 	int r, g, b, a;
 	guint64 dividend;
 	guint width, height;
+	gdouble dd;
 	
 	width = gdk_pixbuf_get_width (pixbuf);
 	height = gdk_pixbuf_get_height (pixbuf);
@@ -2373,11 +2416,12 @@ pixbuf_average_value (GdkPixbuf *pixbuf)
 		dividend = height * width;
 		a_total = dividend * 0xFF;
 	}
-	
-	return ((a_total + dividend / 2) / dividend) << 24
-		| ((r_total + dividend / 2) / dividend) << 16
-		| ((g_total + dividend / 2) / dividend) << 8
-		| ((b_total + dividend / 2) / dividend);
+
+	dd = dividend * 0xFF;
+	result->alpha = a_total / dd;
+	result->red = r_total / dd;
+	result->green = g_total / dd;
+	result->blue = b_total / dd;
 }
 
 static GdkPixbuf *
@@ -3077,7 +3121,7 @@ gnome_bg_create_frame_thumbnail (GnomeBG			*bg,
 
 	result = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, dest_width, dest_height);
 
-	draw_color (bg, result, screen);
+	draw_color (bg, result);
 
 	if (bg->placement != G_DESKTOP_BACKGROUND_STYLE_NONE) {
 		thumb = create_img_thumbnail (bg, factory, screen, dest_width, dest_height, frame_num + skipped);
